/*
 * Z_Corr.c
 *
 *  Created on: Oct 29, 2015
 *      Author: stsuyuki2
 *  This function normalizes input data vector relative to a template.
 *  If the data vector is longer than the template, the function performs
 *  interpolation. Otherwise, extrapolation (adding extra data points between
 *  existing points) is performed. In either case, the resulting array is the 
 *  same length as the template.
 *
 *  Inputs
 *  bk: data vector to be stretched/compressed
 *  L: template length
 *  Lk: length of bk
 *  bk_app: resulting stretched/compressed version of bk
 */
#include "pan_tompkins.h"
 
void Stretch_Comp(int32_t* bk, int32_t L, int32_t Lk, int32_t* bk_app )
{
    //interpolation vs. extrapolation check
	int32_t m;

    if (Lk > L)
	{
		int32_t res=100;
		int32_t index_l, index_h, index_comp, index_l_comp;
		for(m=0; m<L; m++)
		{
			index_l=(Lk*m)/L; //floor((Lk/L)*m)
			index_h=(Lk*m)/L+1; //ceil((Lk/L)*m)
			index_comp=(Lk*m*res)/L;
			index_l_comp=(Lk*m/L)*res;
			/*Original equation:
			index=Lk/L*m
			index_l=floor(Lk/L*m)
			bk_app(m) = bk(index_l) + (bk(index_h) - bk(index_l))*(index - index_l);
			The expression index-index_l will always be equal to zero. 
			index_comp=(Lk*m*100)/L
			index_l_comp=(Lk*m)/L*100
			index_comp-index_l_comp is between 0 and 99*/
			bk_app[m]=((bk[index_h]-bk[index_l])*(index_comp-index_l_comp))/res+bk[index_l];
		}
	}
    else
	{
		int32_t indx;
		int32_t k=0;
		static int32_t countArray[T2_max_length];
		array_init(countArray,T2_max_length);
		//interpolation: count the number of values to include between each value of Lk
		for(m=0; m<L-1; m++)
		{
			indx=(Lk*m)/L;
			if (indx<(k+1)&& indx>=k)
			{				
				countArray[k]=countArray[k]+1;
			}
			else
			{
				k=k+1;
				countArray[k]=countArray[k]+1;
			}
		}
		countArray[k-1]++;

		int32_t v=0;
		int32_t res1;
		for(m=0; m<Lk-1; m++)
		{		
			if (countArray[m]!=0)
			{
				//interpolate between Lk
				res1=((bk[m+1]-bk[m])*10)/countArray[m];
				for(k=0; k<countArray[m]; k++)
				{
					bk_app[v+k]=bk[m]+((k-1)*res1)/10;
				}
				v=v+k;
			}
		}
		bk_app[L-1]=bk[Lk-1];
	}
}

