clc;
clear;
clear mex;  %required to clear static variables
close all;

%% Compile Detection Algorithm
mex *.c

%% Load Data

use_dsp_data = false;
if(use_dsp_data)
    test_data = '../raw_data/sensor_dsp/2017-02-09_01.dat';     %file containing raw data
    D = int32(dlmread(test_data,'\n',1,0));
else
    test_data = '../raw_data/mit_bih/100.csv';     %file containing raw data
    D = csvread(test_data);
    D = D(:,2)';                   %CH1
    %D = D(:,3)';                   %CH2
    %D = (D(:,2)' + D(:,3)')/2;      %Average CH1 and CH2

    D = (D - min(D));
    D = int32(1024 * D/max(D));
end

%% Set parameters
fs = 360;
algorithm_buffer_size = 198;
repeat = 5;

%% Initialize variables

ecg_raw = repmat(D, 1, repeat);

T = 1/fs;                           %Sample Period
t = (1:length(ecg_raw))*T;              %time vector
N = length(ecg_raw);                %number of samples

sections = floor(N/algorithm_buffer_size);
N_processed = sections*algorithm_buffer_size;

arrhythmias = zeros(1, sections);
bpf = zeros(1,N_processed);
qrs = zeros(1,N_processed);

%% Run algorithm
for i = 1:sections
    indices = (1 + (i-1)*algorithm_buffer_size):(i*algorithm_buffer_size);
    d = ecg_raw(indices);
    [arrhythmias(i), bpf(indices), peaks, pvc, t1, t2] = arrhythmia_detection(d);
    
    for j = 1:length(peaks)
        if(peaks(j) > 0 && peaks(j) <= N_processed)
            qrs(peaks(j)) = 1;
        end
    end
end

%% Results

filter_corr = xcorr(double(ecg_raw(1:N_processed)),bpf);
[~, filter_delay] = max(filter_corr);
filter_delay = round(length(filter_corr)/2)-filter_delay;
disp(['The BPF lags the raw data by ' num2str(filter_delay) ' samples']);

figure;
plot(t,double(ecg_raw)/max(abs(double(ecg_raw))), t(1:N_processed), bpf/max(abs(bpf)));
title('Prefiltering of ECG Signal');
legend('Raw ECG Signal', 'BPF Filtered Signal');
xlabel('Time (s)');
ylabel('Normalized Magnitude')

figure;
plot(t(1:N_processed), bpf(1:N_processed), t(1:N_processed), qrs*max(double(bpf(1:N_processed))),'linewidth',1);
arrhythmias_detected = sum(arrhythmias);
title('Algorithm QRS Peak Detection')
legend('BPF ECG Signal','Detected Peaks');
xlabel('Time (s)');
ylabel('Magnitude');
zoom

figure;
plot(t1,'linewidth',1.25)
title('Matching Templete T1: Interval Between QRS Onset and Offset');
xlabel('Sample (n)');
ylabel('Magnitude');
xlim([1 length(t1)])

figure;
plot(t2,'linewidth',1.25)
title('Matching Templete T2: RR-Interval');
xlabel('Sample (n)');
ylabel('Magnitude');
xlim([1 length(t2)])