#include "pan_tompkins.h"

//calculates the difference array for x (size is the length of the x array minus 1, or the length of the y array)
void difference(int32_t* x, int32_t* y, int32_t size)
{
	int32_t i;
	for(i=0; i<size; i++)
	{
		y[i]=x[i+1]-x[i];
	}
}

//calculates the mean of an array of a given size
int32_t mean(int32_t* x, int32_t size)
{
	int32_t i;
	int32_t sum;
	sum=0;
	for(i=0; i<size; i++)
	{
		sum+=x[i];
	}
	return((int32_t)(sum/size));
}

//calculates the difference array for the RR-interval function
void difference_9(int32_t* x, int32_t* y)
{
	int32_t i;
	for(i=0; i<RR_size; i++)
	{
		y[i]=x[i+1]-x[i];
	}
}

//calculates the mean of an array of length 8
int32_t mean_8(int32_t* x)
{
	int32_t i, sum;
	sum=0;
	for(i=0; i<RR_size; i++)
	{
		sum+=x[i];
	}
	return(sum/8);
}

//integer implementation of abs()
int32_t abs_ecg(int32_t x)
{
	int32_t result=x;
	if(x<0)
	{
		result=-1*x;
	}
	return result;
}

//finds the maximum of a portion of a circular buffer
int32_t max_2(int32_t* x, int16_t arr_size, int32_t max_size, int32_t curr_loc)
{
	int32_t i, maximum;
	int32_t max_index=0;
	maximum=x[indexWrap(curr_loc,arr_size)];
	
	for(i=1; i<max_size; i++)
	{
		if(x[indexWrap(curr_loc+i,arr_size)]>maximum)
		{
			maximum=x[indexWrap(curr_loc+i,arr_size)];
			max_index=i;
		}
	}
	return (max_index);
}

//finds the maximum of a linear buffer
int32_t max_no_bounds_checking(int32_t* x, int32_t max_size, int32_t curr_loc)
{
	int32_t i, maximum;
	int32_t max_index=0;
	maximum=x[curr_loc];

	for(i=1; i<max_size; i++)
	{
		if(x[curr_loc+i]>maximum)
		{
			maximum=x[curr_loc+i];
			max_index=i;
		}
	}
	return (max_index);
}

//finds the minimum of a linear buffer
int32_t min_2(int32_t* x, int32_t min_size, int32_t curr_loc)
{
	int32_t i, minimum;
	minimum=x[curr_loc];
	int32_t min_index=0;

	for(i=1; i<min_size; i++)
	{
		if(x[curr_loc+i]<minimum)
		{
			minimum=x[curr_loc+i];
			min_index=i;
		}
	}
	return (min_index);
}

//sets all array elements to zero
void array_init(int32_t* arr, int16_t array_size)
{
	int16_t i;
	for(i=0;i<array_size;i++)
	{
		arr[i]=0;
	}
}

//copies all elements of one array into another
void array_copy(int32_t* src, int32_t* dest, int32_t buffer_size)
{
	int32_t i;
	for(i=0;i<buffer_size;i++)
	{
		dest[i]=src[i];
	}
}

//ensures that indexes remain within the bounds of the circular buffer
int32_t indexWrap(int32_t index, int32_t length)
{
	int32_t result=index;
	if(result>=length)
	{
		result = result%length;
	}
	else while(result<0)
	{
		result+=length;
	}
	return result;
}

//find the number of elements of an array until the first zero element is reached
int32_t get_nonzero_length(int32_t* arr,int32_t arr_size)
{
	int32_t i=0;
	while(arr[i]!=0 && i<arr_size)
	{
		i++;
	}

	return i;
}
