#ifndef __PAN_TOMPKINS_H__
#define __PAN_TOMPKINS_H__

#include <stdint.h>
#include <math.h>

#include "../common/rtecg.h"

#ifdef MATLAB_MEX_FILE
#include <mex.h>
#endif

//#define ARM
#ifdef ARM
#include "arm_math.h"
#else
#include "gen_math.h"
#endif

void arrhythmia_detection(int32_t * signal_in_temp, int32_t * detected, int32_t * bpf, int32_t * peak_index_out, int32_t * pvc_index_out
#ifdef MATLAB_MEX_FILE
       , int32_t * t1_db, int32_t * t2_db
#endif
        );

//Constants
#define FS 360
#define FS2	720
#define FS5 1800
#define MS_50 (5*FS)/100
#define MS_75 (75*FS)/1000
#define MS_150 (15*FS)/100
#define MS_200 (2*FS)/10
#define MS_250 (25*FS)/100
#define MS_360 (36*FS)/100
#define MS_400 (4*FS)/10
#define QRS_size 9
#define RR_size QRS_size-1
#define UART_SIZE 5*BUFFER_SIZE+1
#define WAVE_size 2
#define LP_size 13
#define HP_size 33
#define diff_size 5
#define int_size 64
#define peaks_size 32
#define circ_buff_shift 1

#define signal_length_120s 43200
#define T1_max_length 256
#define T2_max_length 512
#define num_traces 1
//Flash-write
void array_init(int32_t*,int16_t);
void array_copy(int32_t*, int32_t*,int32_t);
int32_t indexWrap(int32_t, int32_t);

//Main functions
void preprocessing(int32_t*, int32_t*, int32_t*, int32_t*, int32_t*, int32_t*, int32_t*, int32_t*, int32_t*);
int32_t WAVELET(int32_t*, int32_t);
int32_t LP(int32_t*, int32_t, int32_t*);
int32_t HP(int32_t*, int32_t, int32_t) ;
int32_t DIFFEREN(int32_t*, int32_t) ;
void SQ(int32_t*,int32_t*);
int32_t MOVAVG(int32_t*, int32_t);
void pantompkins(int32_t*, int32_t*, int32_t*, int32_t, int32_t, int32_t*, int32_t*, int32_t*);
int32_t locate_peaks_highpass(int32_t, int32_t*);
int32_t RR_interval(int32_t*, int32_t, int32_t*, int32_t*, int32_t*);
void lookback(int32_t*, int32_t*, int32_t*, int32_t*, int32_t*, int32_t, int32_t, int32_t*, int32_t, int32_t, int32_t, int32_t*);
void is_T_wave(int32_t*,int32_t,int32_t,int32_t*,int32_t*,int32_t*,int32_t*,int32_t,int32_t,int32_t,int32_t*);
void find_QRS(int32_t*,int32_t*,int32_t,int32_t,int32_t*,int32_t*,int32_t*,int32_t,int32_t,int32_t*,int32_t*,int32_t*,int32_t*);

//Template-matching functions
void generate_templates(int32_t*, int32_t*, int32_t, int32_t, int32_t, int32_t*, int32_t*, float32_t*, int32_t*, int32_t*, float32_t*, int32_t*, int32_t*, int32_t*, int32_t, int32_t);
void temp_match(int32_t*, int32_t, int32_t, int32_t*, int32_t, float32_t, int32_t*, int32_t, float32_t, int32_t, int32_t, float32_t*, int32_t*, int32_t*, int32_t*, int32_t*, int32_t);
int32_t length_circ_buffer(int32_t, int32_t);
//void QS_detect(int32_t*, int32_t, int32_t*, int32_t*, int32_t*, int32_t*);
void Stretch_Comp(int32_t*, int32_t, int32_t, int32_t*);
float32_t Corr_Coeff(int32_t*, int32_t*, float32_t, int32_t);
float32_t Z_Corr(float32_t, float32_t, int32_t);

//Helper functions
void difference(int32_t*, int32_t*, int32_t);
int32_t mean(int32_t*, int32_t);
void difference_9(int32_t*,int32_t*);
int32_t mean_8(int32_t*);
int32_t abs_ecg(int32_t);
int32_t max_2(int32_t*, int16_t, int32_t, int32_t);
int32_t max_no_bounds_checking(int32_t*, int32_t, int32_t);
int32_t min_2(int32_t*, int32_t, int32_t);
int32_t get_nonzero_length(int32_t*,int32_t);

#endif
