#include "pan_tompkins.h"

void preprocessing(int32_t* signal_in, int32_t* highpass, int32_t* signal_temp, int32_t* wavelet, int32_t* wavelet_buff, int32_t* lp_circ_buff, int32_t* hp_circ_buff, int32_t* diff_circ_buff, int32_t* int_circ_buff)
{
	static int32_t lp_prev[2]={0,0}; //{y(n-1) y(n-2)}
	static int32_t hp_prev=0; //y(n-1)
	static int32_t wave_index=0;
	static int32_t lp_index=0;
	static int32_t hp_index=0;
	static int32_t diff_index=0;
	static int32_t int_index=0;
	int32_t j;
	
	for(j=0; j<SENSOR_BUFFER; j++)
	{
		lp_circ_buff[lp_index]=signal_in[j];
		signal_temp[j]=LP(lp_circ_buff,lp_index,lp_prev);
		lp_prev[1]=lp_prev[0];
		lp_prev[0]=signal_temp[j];
		lp_index=indexWrap(lp_index-1,LP_size);
		
		wavelet_buff[wave_index]=signal_in[j];
		wavelet[j]=WAVELET(wavelet_buff,wave_index);
		wave_index=indexWrap(wave_index-1,WAVE_size);
	}
	for(j=0; j<SENSOR_BUFFER; j++)
	{
		hp_circ_buff[hp_index]=signal_temp[j];
		signal_in[j]=HP(hp_circ_buff,hp_index,hp_prev);
		hp_prev=signal_in[j];
		hp_index=indexWrap(hp_index-1,HP_size);
	}
	array_copy(signal_in,highpass,SENSOR_BUFFER);
	for(j=0; j<SENSOR_BUFFER; j++)
	{
		diff_circ_buff[diff_index]=signal_in[j];
		signal_temp[j]=DIFFEREN(diff_circ_buff,diff_index);
		diff_index=indexWrap(diff_index-1,diff_size);
	}
	SQ(signal_temp,signal_in);
	for(j=0; j<SENSOR_BUFFER; j++)
	{
		int_circ_buff[int_index]=signal_in[j];
		signal_temp[j]=MOVAVG(int_circ_buff,int_index);
		int_index=indexWrap(int_index-1,int_size);
	}	
}

int32_t WAVELET(int32_t* filt_input, int32_t index)
{
	//(1/1.5)*(-2 x(n) + 2 x(n - 1))
	int32_t result;
	result=(int32_t)(((2.0f)*(-2*(float32_t)filt_input[indexWrap(index,WAVE_size)]+2*(float32_t)filt_input[indexWrap(index+1,WAVE_size)]))/(3.0f));
	return result;
}

int32_t LP(int32_t* filt_input, int32_t index, int32_t* filt_prev)
{
	//2 y(n-1) - y(n-2) + x(n) - 2 x(n - 6) + x(n - 12)
	int32_t result;
	result=(2*filt_prev[0])-filt_prev[1]+filt_input[indexWrap(index,LP_size)]-(2*filt_input[indexWrap(index+6,LP_size)])+filt_input[indexWrap(index+12,LP_size)];
	return result;
}

int32_t HP(int32_t* filt_input, int32_t index, int32_t hp_prev)
{
	//y(n-1) - x(n)/32 + x(n - 16) - x(n - 17) + x(n - 32)/32
	int32_t result;
	result=hp_prev-(filt_input[indexWrap(index,HP_size)]/32)+filt_input[indexWrap(index+16,HP_size)]-filt_input[indexWrap(index+17,HP_size)]+(filt_input[indexWrap(index+32,HP_size)]/32);
	return result;
}

int32_t DIFFEREN(int32_t* filt_input, int32_t index)
{
	//(2 x(n))/8 + x(n - 1)/8 - x(n - 3)/8 - (2 x(n - 4))/8
	int32_t result;
	result=(2*filt_input[indexWrap(index,diff_size)]/8)+(filt_input[indexWrap(index+1,diff_size)]/8)-(filt_input[indexWrap(index+3,diff_size)]/8)-(2*filt_input[indexWrap(index+4,diff_size)]/8);
	return result;
}

void SQ(int32_t* x, int32_t* y)
{
  	uint32_t m;
   	int32_t temp;

   	for(m=0; m<SENSOR_BUFFER; m++)
   	{
   		temp=(int32_t)x[m]*x[m];
   		y[m]=(int32_t)temp;
   	}
}

int32_t MOVAVG(int32_t* filt_input, int32_t index)
{
   	uint32_t n;
	int32_t sum = 0;

	for(n=0; n<64; n++)
	{
		sum = sum+filt_input[indexWrap(index+n,int_size)];
	}
	return ((int32_t)(sum/64));
}
