/* Real-Time Electrocardiogram Monitoring
   Common code
   rtecg.h - Project-wide constants and structures.
   This file is used by ti_c5515, comm_daemon, acquisition_daemon, and 
     avr-testing
   Calvin Walden and Edward Sandor
   2017-01-31
*/

#ifndef RTECG_H
#define RTECG_H

/* Buffer sizes */
#define SENSOR_BUFFER 198 /* Samples for 1 second */
#define BUFFER_SIZE (25 * SENSOR_BUFFER)
/* BUFFER_SIZE: Local buffer size for rtecg_aquisition. When full, the buffer
   is written to a file. */

/* PATH_MAX equivalent for our project. */
#define PROJECT_PATH_MAX    2048
/* I've defined this and we're going with it instead of messing with the
   finicky nature of PATH_MAX on various POSIX standards 
   (see: http://stackoverflow.com/questions/833291) */

/* I2C addresses (depricated) */
#define DSP_ADDRESS         0x08
#define DSP_SIM_ADDRESS     0x06
#define MASTER_ADDRESS      0x00

/* Serial configuration */
#define SERIAL_BAUD         57600
#define SERIAL_DEVICE       "/dev/serial0"
#define MAX_ARGUMENT_COUNT  255

/* Serial packet header */
#define PACKET_CONST_H 0xFF
#define PACKET_CONST_L 0xFF
#define PACKET_CONST (PACKET_CONST_H << 8) | PACKET_CONST_L
#define MAKE_PACKET_HEADER(command, argument_count) \
    ((PACKET_CONST << 16) | command << 8 | argument_count)

/* Serial data flags */
#define FLAG_CONST          0x55
/* This constant flag is to be combined with the others to generate a distinct
   control pattern, such as FLAG_START = 0x1155
   The macro MAKE_FLAG will do this:
*/
#define MAKE_FLAG(flag) ((flag << 8) | FLAG_CONST)

#define FLAG_ARRHYTHMIA_DETECTED  0x11 /* Flag to signal an arrhythmia has been 
                                          detected by the algorithm */
#define FLAG_START                0x22
#define FLAG_SEND_PVC             0x33 /* Flag for sending PVC peak indicies */
#define FLAG_SEND_QRS             0x44 /* Flag for sending QRS peak indicies */
#define FLAG_SEND_FILT            0x66 /* Flag for sending filtered ECG data */
#define FLAG_SEND_RAW             0x77 /* Flag for sending raw ECG data */
#define FLAG_END_BUFFER           0xEE /* Flag at end of buffer transmission */
#define FLAG_STOP                 0xFF

/* DSP Commands */
#define CMD_RESET                       0xFF
//#define CMD_START                     0x01
#define CMD_STOP                        0x02
#define CMD_ACQUIRE_ECG                 0x03    /* specifiy data source instead
                                                   of CMD_START */
#define CMD_ACQUIRE_BENCHMARK_DATA      0x04
#define CMD_BENCHMARK_DATA              0x0B
#define CMD_POLL                        0x10
#define CMD_ACQUIREHEADER               0x20
#define CMD_ACQUIREPAYLOAD              0x30
#define DSP_POLL_READY                  0x01
#define DSP_POLL_NOTREADY               0x00

#define DSP_POLL_READY            0x01
#define DSP_POLL_NOTREADY         0x00

#endif /* RTECG_H */

