%% Real-Time ECG PVC Detection Algorithm Tester
%This script simulates the algorithm to be used for the Real-Time ECG
%processing senior project
%Edward Sandor

clear;
close all;

%% Load Data

test_data = '../raw_data/sensor_dsp/2017-02-09_01.dat';     %file containing raw data


D = int32(dlmread(test_data,'\n',1,0)');
fs = 360;
ecg_raw = D;

%D = csvread(test_data);
%ecg_raw = D(:,2);                   %CH1
%ecg_raw = D(:,3);                   %CH2
%ecg_raw = (D(:,2) + D(:,3))/2;      %Average CH1 and CH2

%% Initialization
fs = 360;                           %Sample Frequency
T = 1/fs;                           %Sample Period
t = (1:length(D))'*T;              %time vector
N = length(ecg_raw);                %number of samples
MA = 64;                            %number of terms for moving average
S1 = 1/T;
S2 = 2/T;
S5 = 5/T;

%Threshold values used to detect QRS
SPKI = int32(0);
NPKI = int32(0);
threshold_I1 = int32(0);
threshold_I2 = int32(0);
SPKF = int32(0);
NPKF = int32(0);
threshold_F1 = int32(0);
threshold_F2 = int32(0);
f_peak = 1;
f_peak_1 = 1;
f_peak_2 = 1;

%RR Thresholds (RR1 is most recent RR, RR2 is most recent within limits)
RR_average_1  = int32(0);
RR_average_2  = int32(0);
RR_previous_1 = int32(zeros(8,1));
RR_previous_2 = int32(zeros(8,1));
last_peak = int32(0);

blockout = 0;
blockout_count = 0;
blockout_length = 0.2/T;

%Keep a history of the thresholds for debugging only
threshold_I1_deb = zeros(N,1);
threshold_I2_deb = zeros(N,1);
threshold_F1_deb = zeros(N,1);
threshold_F2_deb = zeros(N,1);

%Allocate space for Pan-Tompkins intermediate steps
ecg_lpf = int32(zeros(N,1));
ecg_hpf = int32(zeros(N,1));
ecg_diff = int32(zeros(N,1));
ecg_sq = int32(zeros(N,1));
ecg_i = int32(zeros(N,1));
ecg_qrs = int32(zeros(N,1));

%lpf initial values
ecg_lpf(1) = ecg_raw(1);
ecg_lpf(2) = 2*ecg_lpf(1) + ecg_raw(2);
%hpf initial values
ecg_hpf(1) = -ecg_lpf(1)/32;
%differentiator initial values
ecg_diff(1) = (2*ecg_hpf(1))/8;
ecg_diff(2) = (2*ecg_hpf(2) + ecg_hpf(1))/8;
ecg_diff(3) = (2*ecg_hpf(3) + ecg_hpf(2))/8;
ecg_diff(4) = (2*ecg_hpf(4) + ecg_hpf(3) - ecg_hpf(1))/8;

%% Algorithm

for n = 1:N
    %% Pan-Tompkins QRS Detection
    %% Prefiltering
    %LPF
    if(n < 13)
        if(n < 7)
            if(n > 2)
                ecg_lpf(n) = 2*ecg_lpf(n-1) - ecg_lpf(n-2) + ecg_raw(n);
            end
        else
            ecg_lpf(n) = 2*ecg_lpf(n-1) - ecg_lpf(n-2) + ecg_raw(n) - 2*ecg_raw(n-6);
        end
    else
        ecg_lpf(n) = 2*ecg_lpf(n-1) - ecg_lpf(n-2) + ecg_raw(n) - 2*ecg_raw(n-6) + ecg_raw(n-12);
    end
    %HPF
    if(n < 33)
        if(n < 18)
            if(n < 17)
                if(n > 1)
                    ecg_hpf(n) =  ecg_hpf(n-1) - ecg_lpf(n)/32;
                end
            else
                ecg_hpf(n) = ecg_hpf(n-1) - ecg_lpf(n)/32 + ecg_lpf(n-16);
            end
        else
            ecg_hpf(n) = ecg_hpf(n-1) - ecg_lpf(n)/32 + ecg_lpf(n-16) - ecg_lpf(n-17);
        end
    else
        ecg_hpf(n) = ecg_hpf(n-1) - ecg_lpf(n)/32 + ecg_lpf(n-16) - ecg_lpf(n-17) + ecg_lpf(n-32)/32;
    end
    %Differentiator
    if(n > 4)
        ecg_diff(n) = (2*ecg_hpf(n) + ecg_hpf(n-1) - ecg_hpf(n-3) - 2*ecg_hpf(n-4))/8;
    end
    %Square
    ecg_sq(n) = ecg_diff(n) * ecg_diff(n);
    %moving average integrator
    ma_sum = 0;
    for i = 1:MA
        if((n-i) > 0)
            ma_sum = ma_sum + ecg_sq(n-i);
        end
    end
    ecg_i(n) = ma_sum/MA;
    
    %% Find last peak in hpf signal (greatest of previous two peaks)
    if(n > 2)
        %Detect local maximum if n-1 is greater than each neighbor point
        if((ecg_hpf(n) <= ecg_hpf(n-1) && ecg_hpf(n-2) < ecg_hpf(n-1)) || (ecg_hpf(n) < ecg_hpf(n-1) && ecg_hpf(n-2) <= ecg_hpf(n-1)))
            f_peak_2 = f_peak_1;
            f_peak_1 = n-1;
            
            if(ecg_hpf(f_peak_1) > ecg_hpf(f_peak_2))
                f_peak = f_peak_1;
            else
                f_peak = f_peak_2;
            end
        end
    end
    
    if(n > S2)
        %% Normal Processing
        %Detect local maximum if n-1 is greater than each neighbor point
        if((ecg_i(n) < ecg_i(n-1) && ecg_i(n-2) <= ecg_i(n-1)) || (ecg_i(n) <= ecg_i(n-1) && ecg_i(n-2) < ecg_i(n-1)))
            if(blockout == 0 && ecg_i(n-1) > threshold_I1 && ecg_hpf(f_peak) > threshold_F1)
                %peak detected
                ecg_qrs(f_peak) = 1;
                %adjust thresholds
                SPKI = ecg_i(n-1)/8 + 7*SPKI/8;  
                SPKF = ecg_hpf(f_peak)/8 + 7*SPKF/8;   
                blockout = 1;
                %update RR
                RR = n-1 - last_peak;
                if(last_peak ~= 0)
                    RR_previous_1 = [RR_previous_1(2:8); RR];
                    if(RR_average_2 == 0 && RR_previous_1(8))
                        %%Learning phase 2
                        RR_average_1 = int32(mean([RR_previous_1(8) RR]));
                        RR_previous_2 = int32(ones(8,1))*RR_average_1;
                    end
                end
                %set RR thresholds
                if(min(RR_previous_1) > RR_average_2*0.92 && max(RR_previous_1) < RR_average_2*1.16)
                    RR_average_1 = int32(mean(RR_previous_1));
                    RR_previous_2 = int32(ones(8,1))*RR_average_1;
                elseif(RR > RR_average_2*0.92 && RR < RR_average_2*1.16)
                    RR_previous_2 = [RR_previous_2(2:8); RR];
                end
                RR_average_2 = int32(mean(RR_previous_2));
                last_peak = n-1;
                
            elseif(blockout == 0)    
                %noise detected (not blocked out)
                NPKI = ecg_i(n-1)/8 + 7*NPKI/8; 
                NPKF = ecg_hpf(f_peak)/8 + 7*NPKF/8;   
            end

            %update thresholds
            threshold_I1 = NPKI + ((SPKI - NPKI)/4);
            threshold_I2 = threshold_I1/4;
            threshold_F1 = NPKF + ((SPKF-NPKF)/4);
            threshold_F2 = threshold_F1/2;
        end
    elseif(n == S2)
        %% Learning Phase 1
        %Set initial values by checking maximum values of first 2 seconds
        SPKI = max(ecg_i(1:n));
        NPKI = max(ecg_i(1:n));
        threshold_I1 = NPKI + ((SPKI - NPKI)/4);
        threshold_I2 = threshold_I1/4;
        SPKF = max(ecg_hpf(1:n));
        NPKF = 0;
        threshold_F1 = NPKF + ((SPKF-NPKF)/4);
        threshold_F2 = threshold_F1/2;
    end
    
    if(blockout == 1 && blockout_count < blockout_length)
        blockout_count = blockout_count + 1;
    else
        blockout = 0;
        blockout_count = 0;
    end
        
    threshold_I1_deb(n) = threshold_I1;
    threshold_I2_deb(n) = threshold_I2;
    threshold_F1_deb(n) = threshold_F1;
    threshold_F2_deb(n) = threshold_F2;
end

%% Plot Outputs
figure;
plot(t, ecg_raw);
figure;
plot(t, ecg_lpf);
figure;
plot(t, ecg_hpf);
figure;
plot(t, ecg_hpf, t, ecg_qrs*max(ecg_hpf), t, threshold_F1_deb, t, threshold_F2_deb);%, t, threshold_F1_deb, t, threshold_F2_deb);
grid on;
title('Pan Tompkins QRS Detection Performance');
xlabel('time (s)');
ylabel('amplitude');
legend('Filtered ECG', 'Detected QRS');
figure;
plot(t, ecg_diff);
figure;
plot(t, ecg_sq);
figure;
plot(t, ecg_i);
figure;
plot(t, ecg_i, t, ecg_qrs*max(ecg_i), t, threshold_I1_deb, t, threshold_I2_deb);
figure;
plot(t, ecg_qrs);