/* Real-Time Electrocardiogram Monitoring
   Acquisition Daemon
   file.h - ECG data log file writing and creation operations
   Calvin Walden
   2017-03-20
*/

#ifndef FILE_H
#define FILE_H

#include <time.h>
#include <stdint.h>

#include "file.h"

extern struct logspec logspec_filt;
extern struct logspec logspec_raw;

/* Specification of log file */
struct logspec {
    char filename_current[1024]; /* Current filename, modified by 
                                    fileMakeFilename() and used by
                                    fileWriteLog() */
    char filename_base[100];     /* Base of filename, such as "filtered_data"
                                    or "raw_data". It will be concatendated 
                                    with a timestamp and extension of ".log". */
    char name[1024];             /* Human-readable name */
    time_t last_modified;        /* Last modification time of filename_current, 
                                    checked and modified by 
                                    fileMakeFilename(). */
};



/* fileInitializeLogspecs(): Initialize the logspec structures with default 
   values defined above.
   Input: None
   Output: None
   Return: None
*/
int fileInitializeLogspecs(void);



/* fileMakeFilename(): Using the current time, the last modified time of the 
   given logfile (logspec->last_modified), and the basename
   (logspec->filename_base), either generate a new filename if enough time has 
   elapsed (FILE_HISTORY in acquire.h), or keep the current one.
   Input: logfile (multiple elements)
   Output: logfile->filename_current
   Return: None
*/
int fileMakeFilename(struct logspec* logfile);



/* fileWriteLog(): Write the contents of buffer to the logspec's current file.
   This is called by acquireDump() when the buffer is determined to be full.
   Input: buffer - buffer to write to log file, first column
          peak_buffer - supplementary peak indicies, second column
          peak_len - length of peak_buffer
          logfile - specification for logfile, provides filename
*/
int fileWriteLog(int16_t* buffer,
                 uint16_t* peak_buffer, uint16_t peak_len,
                 struct logspec* logfile);


    
#endif /* FILE_H */

