/* Real-Time Electrocardiogram Monitoring
   Acquisition Daemon
   main.c - Main source file
   Calvin Walden
   2017-03-20
*/

#include <stdlib.h>
#include <unistd.h>
#include <syslog.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>

#include "../../common/config.h"
#include "serial.h"
#include "acquire.h"

int main(int argc, char *argv[]) {
    pid_t pid;
    pid_t sid;
    int ret;

    pid = fork();

    if (pid < 0) {
        exit(EXIT_FAILURE);
        /* Unable to fork() for some reason. */
    } else if (pid > 0) {
        exit(EXIT_SUCCESS);
    }

    /* At this point, this is the child process. */
    umask(0);

    /* Open up logfile */
    openlog(argv[0], LOG_NOWAIT|LOG_PID, LOG_USER);

    /* First syslog message */
    syslog(LOG_NOTICE, "Started Real-Time ECG Acquisition Daemon.");

    /* Get our own session ID */
    sid = setsid();
    if (sid < 0) {
        syslog(LOG_ERR, "main: Could not create process group.");
        exit(EXIT_FAILURE);
    }

    /* Load configuration */
    ret = configLoad();
    if (ret) {
        syslog(LOG_ERR, "main: Error at configLoad(). Exiting.");
        exit(EXIT_FAILURE);
    }

    /* Close stdio file descriptors, since we don't really need them. */
#if 0
    /* EDIT: This was blocked out because it was causing problems with 
       manipulating file descriptors later on - getting lots of 
       "invalid file descriptor errors" with fd = 0, 1, 2. */
    close(STDIN_FILENO);
    close(STDOUT_FILENO);
    close(STDERR_FILENO);
#endif

    /* Set up serial port */
    ret = serialSetup();
    if (ret) {
        syslog(LOG_NOTICE, "main: Error at serialSetup().");
        return 1;
    }

    /* Set up logfile specifications */
    fileInitializeLogspecs();

    /* Main loop - read data from serial port */
    acquireLoop();

    /* ...And we're done here. */
    serialClose();
    closelog();

    return 0;
}
