/* Real-Time Electrocardiogram Monitoring
   Acquisition Daemon
   serial.h - Serial communication functions
   Calvin Walden
   2017-03-20
*/

#ifndef SERIAL_H
#define SERIAL_H

#include <stdint.h>

/* #define DEBUG_SERIAL */



/* serialSetup(): Initialize the serial port, SERIAL_DEVICE in rtecg.h, to use
   SERIAL_BAUD with 8n1 protocol.
   Input: None
   Output: None
   Return: 0 on success, 1 on failure

   Major reference: http://stackoverflow.com/questions/6947413 
*/
int serialSetup(void);



/* serialClose(): Close the serial port. */
int serialClose(void);



/* serialRead(): Read a string, of length len, of bytes into buffer
   Input: buffer, len
   Output: buffer
   Return: 0 on success (len bytes read), 1 on failure (less than len bytes 
   read)
 */
int serialRead(uint8_t* buffer, int len);



/* serialRead16(): Read a single signed 16-bit word from the serial port. If 
   two bytes are not available in the system's buffer, wait for both of them. 
   This function will block the running thread if this happens.
   Expected byte order: High byte first, low byte second
   Input: None
   Output: None
   Return: Read value, or 0 on failure (accompanied by error message in syslog)
*/
int16_t serialRead16(void);



/* serialRead8(): Read a single, unsigned byte from the serial port.
   Input: None
   Output: None
   Return: Read value, or 0 if a byte could not be read.
*/
uint8_t serialRead8(void);



/* serialTranslateBaud(): Helper function that returns the "B----" constant 
   from <termbits.h> corresponding to integer_baud that cfsetospeed() wants. 
   Input: integer_baud
   Output: None
   Return: <termbits.h> constant, or -1 if undefined baud is specified
*/
int serialTranslateBaud(int integer_baud);



#endif /* SERIAL_H */

