/* Real-Time Electrocardiogram Monitoring
   Communication Daemon
   comm.h - Communication functions, including main loop
   Calvin Walden
   2017-03-20
*/

#ifndef COMM_H
#define COMM_H

#include "comm.h"

/* #define DEBUG_COMM */



/* commLoop(): Main loop for communication daemon. The loop polls for flags
   set by the signal handler function, all found in signalhandler.h 
   Input: None
   Output: None
*/
int commLoop(void);



/* commTransmit(): Transmit last 15 minutes of data, probably routine and 
   called by cron(8) 
   Output: None
   Input: None
   Return: 0 on success, 1 on failure
*/
int commTransmit(void);



/* commTransmitUrgent(): Urgent data transmission, most likely following an 
   arrhythmia being detected by the DSP and logged by rtecg_acquisition(1).
   This is roughly the same as commTransmit() with some different strings,
   but its separation allows for more functionality in the event of an
   emergency.
   Input: None
   Output: None
   Return: 0 on success, 1 on failure
*/
int commTransmitUrgent(void);



/* commGetNewestLogfile(): Get the filename (not path) of the newest logfile 
   in the project's logfile directory, probably /var/rtecg or something like 
   that. This is done by getting a directory listing sorted alphabetically,
   and filtering out all files but those matching *.log.
   Input: None
   Output: filename
   Return: 0 on success, 1 on failure
*/
int commGetNewestLogfile(char* filename);


#endif /* COMM_H */
