/* Real-Time Electrocardiogram Monitoring
   Communication Daemon
   mail.h - Functionality for sending email messages
   Calvin Walden
   2017-04-16
*/

#ifndef MAIL_H
#define MAIL_H

#include <stdarg.h>

#include "mail.h"

/* #define DEBUG_MAIL */



/* sendMail: Send an email to a single recipient, with subject and body by 
   calling mail(1) using system(3).
   Input: recipient: Recipient's email address
          subject: Subject of email
          body: Body text of email
   Output: None
   Return: 0 on success, 1 on failure

   This function may block for a long period of time ( > 1 second) while
   mail(1) sends the message.
*/
int sendMail(char* recipient, char* subject, char* body);



/* sendMailStartup(): Send a system startup notification message using
   sendMail(). This message contains information about  system configuration
   (assigned name, ID, etc) and system status (interface IP addresses and 
   filesystem free space).
   Input: None
   Output: None
   Return value: 0 on success, 1 on failure
 */
int sendMailStartup(void);



/* sendMailAttachments: Send an email to a single recipient, with subject, body, and
   an indeterminate number of attachments by calling mail(1) using system(3).
   Input: recipient: Recipient's email address
          subject: Subject of email
          body: Body text of email
          num_attachments: Number of attachments to follow
          ...: List of char*s of absolute paths of files to include.
   Output: None
   Return: 0 on success, 1 on failure

   Just like sendMail(), this function may block for a long period of 
   time ( > 1 second) while mail(1) sends the message.

   As it is currently implemented, this function does no sanity checking on 
   the attachment paths, it just inserts them into the command string passed 
   to system(3).
*/
int sendMailAttachments(char* recipient, char* subject, char* body,
                        int num_attachments, ...);



#endif /* MAIL_H */
