/* Real-Time Electrocardiogram Monitoring
   Communication Daemon
   net.h - Functions for getting network interface status
   Calvin Walden
   2017-03-20
*/

#ifndef NET_H
#define NET_H

#include "net.h"

/* #define DEBUG_NET */
#define NET_IP_LENGTH 15 /* XXX.XXX.XXX.XXX max */
#define NET_SLEEP_TIME 5 /* seconds, interval between attempts */
#define NET_MAX_ATTEMPTS 60



/* netGetIP: Given the name of a network interface, get its IPv4 address.
   Input: interface (probably "eth0" or "wlan0")
          ip_len - length of string char* interface
   Output: ip
   Return: 0 on success, 1 on failure

   Based loosely on code found here:
   http://stackoverflow.com/questions/2283494/
 */
int netGetIP(char* interface, char* ip, int ip_len);



/* netWaitForInterface: Using the same mechanism as netGetIP, wait for an 
   interface to come online, a condition defined here as being assigned
   an IP address by DHCP. This function will try up to NET_MAX_ATTEMPTS to
   read the assigned IP address of the interface, waiting NET_SLEEP_TIME 
   seconds before attempts. During this time, the function blocks the thread
   it is running in.
   Input: interface - name of network interface
   Output: None
   Return: 0 on success (interface had/acquired IP address), 1 on failure
*/
int netWaitForInterface(char* interface);



#endif /* NET_H */

