/* Real-Time Electrocardiogram Monitoring
   Communication Daemon
   signalhandler.h - User signal handler and its setup function
   Calvin Walden
   2017-03-20
*/

#include <signal.h>
#include <stdio.h>
#include <syslog.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>

#include "signalhandler.h"

/* As per <signal.h> */
typedef void (*sighandler_t)(int);

/* Global variables for signal handler status */
int sigusr1_flag = 0;
int sigusr2_flag = 0;



/* signalHandlerSetup(): Register handler, signalHandler(), for SIGUSR1 and 
   SIGUSR2 with the kernel
   Input: None
   Output: None
   Return: 0 on success, 1 on failure
*/
int signalHandlerSetup(void)
{
    sighandler_t ret;
    char msg_buffer[200];

    ret = signal(SIGUSR1, signalHandler);
    if (ret == SIG_ERR) {
        snprintf(msg_buffer, sizeof(msg_buffer),
                 "main: Could not assign handler for SIGUSR1: %s",
                 strerror(errno));
        syslog(LOG_ERR, msg_buffer);
        return 1;
    }

    ret = signal(SIGUSR2, signalHandler);
    if (ret == SIG_ERR) {
        snprintf(msg_buffer, sizeof(msg_buffer),
                 "main: Could not assign handler for SIGUSR2: %s",
                 strerror(errno));
        syslog(LOG_ERR, msg_buffer);
        return 1;
    }

    return 0;
}



/* signalHandler(): This is called when SIGUSR1 or SIGUSR2 is received by the 
   daemon process. In both cases, the respective flag, a global variable, is 
   set, and the desired tasks are carried out by commLoop().
   Input: signal - number of the signal, defined in <signal.h>
   Output: None
   Return: None
*/
void signalHandler(int signal)
{
    switch(signal) {
    case SIGUSR1:
        sigusr1_flag = 1;
        break;
    case SIGUSR2:
        sigusr2_flag = 1;
        break;
    default:
        break;
    }

    /* Reset, because apparently we have to do this. */
    signalHandlerSetup();
            
    return;
}

