/* Real-Time Electrocardiogram Monitoring
   Shutdown button daemon rtecg_shutdown
   Calvin Walden
   2017-01-15

   This program monitors a specified GPIO pin connected to a designated
   shutdown pushbutton. When the button is pressed, the program invokes a
   clean system shutdown.
*/

#include <stdlib.h>
#include <stdio.h>
#include <wiringPi.h>
#include <unistd.h>

#include "rtecg_shutdown.h"

int shutdown_flag = 0;


int main(void)
{
    wiringPiSetupGpio(); 

    printf("\nshutdown_button: Monitoring for shutdown button press"
           " on GPIO pin %i\n", SHUTDOWN_BUTTON);

    wiringPiISR(SHUTDOWN_BUTTON, INT_EDGE_FALLING, handlePinInterrupt);

    while(shutdown_flag == 0) { sleep(1); }

    return 0;
}



void handlePinInterrupt(void)
{
    int ret;

    printf("\nshutdown_button: Button pressed. Shutting down Raspberry Pi.\n");

    shutdown_flag = 1;

    ret = system("/sbin/shutdown -hP now &");
    if (ret) {
        printf("system() returned %i", ret);
        fflush(stdin);
        getchar();
    }

    return;
}

