#include <stdio.h>

#include "gpio.h"

#include "lcd.h"

#include "button.h"

/** Bit mask for the GPIO buttons */
#define BTN_MASK 0x1F
#define DEB_BITS 0b1111111111111111

int debounce_up = 0;
int debounce_down = 0;
int debounce_right = 0;
int debounce_left = 0;
int debounce_sel = 0;



/**
 * @brief Convert a raw value to a button enum value.
 * @warning may not be working due to lcdproc compatibility changes
 */
static Button raw_to_button(uint8_t btn)
{
    switch (~btn) {
    case ~1:
        return Select;
    case ~2:
        return Right;
    case ~4:
        return Down;
    case ~8:
        return Up;
    case ~16:
        return Left;
    default:
        return Null;
    }
}

uint8_t btn_nblk_raw()
{
    return GPIO_read(PortA) & BTN_MASK;
}

Button btn_nblk()
{
    return raw_to_button(btn_nblk_raw());
}

uint8_t btn_blk_raw()
{
    uint8_t buf;
    while ((buf = btn_nblk_raw()) == 0);
    return buf;
}

Button btn_blk()
{
    return raw_to_button(btn_blk_raw());
}

Button btn_return_clk()
{
    uint8_t buf;
    while ((buf = btn_nblk_raw()) == 0);
    while (btn_nblk_raw() != 0);
    return buf;
}

int btn_printf(Button button)
{
    switch (button) {
    case Select:
        return printf("Select\n");
        break;
    case Right:
        return printf("Right\n");
        break;
    case Down:
        return printf("Down\n");
        break;
    case Up:
        return printf("Up\n");
        break;
    case Left:
        return printf("Left\n");
        break;
    default:
        return printf("Null\n");
        break;
    }
}


int getButtonPress()
{
	switch(btn_nblk_raw())
		{
			case 15: /* LEFT */
				debounce_left <<= 1;
				debounce_left |= 1;
				if(debounce_left == DEB_BITS)
				{
					return Left;
					
				}
				break;
			
			case 23: /* UP */
				debounce_up <<= 1;
				debounce_up |= 1;
				if(debounce_up == DEB_BITS)
				{
					return Up;
				}
				break;
			
			case 27: /* DOWN */
				debounce_down <<= 1;
				debounce_down |= 1;
				if(debounce_down == DEB_BITS)
				{
					return Down;
				}
				break;
			
			case 29: /* RIGHT */
				debounce_right <<= 1;
				debounce_right |= 1;
				if(debounce_right == DEB_BITS)
				{
					return Right;
				}
				break;
			
			case 30: /* SELECT */
				debounce_sel <<= 1;
				debounce_sel |= 1;
				if(debounce_sel == DEB_BITS)
				{
					return Select;
				}
				break;
			
			default:
				debounce_up = 0;
				debounce_down = 0;
				debounce_sel = 0;
				debounce_left = 0;
				debounce_right = 0;
				return 0;
				
		}
}