#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>

#include "menu.h"
#include "button.h"
#include "gpio.h"
#include "lcd.h"
#include "lcd_cgram.h"
#include "signal.h"

//for IP
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <net/if.h>
#include <arpa/inet.h>

//for pipe
#include <fcntl.h>
#include <sys/stat.h>
#include <unistd.h>


#define INTERFACE "wlan0"
//#define DEB_BITS 0b1111111111111111

static int set_color(const char *s);
static int set_cursor(const char *s);

const int MAX_COUNT = 333333333;

int timer = 0;


int main(int argc, char *const *argv)
{
    int r = 0;
    int s = 0;
    int opts_index = 0;

    int lcd_off = 0;
    int verbose = 0;
	
	/* Initialize GPIO and LCD */
    GPIO_open();
    LCD_init(0);
	
	/* Initialize LCD Settings */
	char color = 7;
	LCD_color(color);
	set_cursor("Off");
	verbose = 0;
	
	/* Create Menu Items */
	struct menu_item item_main = {"main", "Real-time ECG   Monitoring", 0, NULL, NULL};
	struct menu_item item_bpm = {"bpm", "Current BPM: ", 0, NULL, NULL};
	struct menu_item item_ip = {"ip", "Config Address  ", 0, NULL, NULL};
	struct menu_item item_color = {"color", "Select Color          <  >", 0, NULL, NULL};
	struct menu_item item_reboot = {"power", "Power Options:  ", 0, NULL, NULL};
	struct menu_item item_sendsig = {"sendsig", "Simulate Email:     [ SEND ]", 0, NULL, NULL};
	
	struct menu_item dev_menu[6] = {item_main, item_bpm, item_ip, item_color, item_reboot, item_sendsig};
	
	int item = 0; //*p_dev_menu = &dev_menu[0];
	int temp_item = 0;
	
	int but = 0;
	int counter = 0;
	int refreshCounter = 0;
	
	/*
	int debounce_up = 0;
	int debounce_down = 0;
	int debounce_right = 0;
	int debounce_left = 0;
	int debounce_sel = 0;
	*/
	
	int init = 0;
	int lcdUpdate = 0;
	
	char powerOption = 0;
	char * power = ">Reboot      Off";
	
	
	int fd;
	struct ifreq ifr;
	char display_text[32];
	char * bpm_pipe = "/tmp/bpm";
	char * buf;
	char lcdcolor;
	char clr;
	
	
    while(1)
    {
		
		switch(getButtonPress())
		{
			case Left: /* LEFT */
				if(item == 4)
				{
					powerOption = 0;
					power = ">Reboot      Off";
					lcdUpdate = 1;
				}
				else if(item == 3)
				{
					color--;
					if(color < 0)
					{
						color = 0;
					}
					//LCD_color(color);
				}
				
				LCD_color(color);
				timer = 0;
				break;
			
			case Up: /* UP */
				
				temp_item--;
				if(temp_item < 0)
				{
					temp_item = 5;
				}
				
				LCD_color(color);
				timer = 0;
				break;
			
			case Down: /* DOWN */
				
				temp_item++;
				if(temp_item > 5)
				{
					temp_item = 0;
				}
				
				LCD_color(color);
				timer = 0;
				break;
			
			case Right: /* RIGHT */
				
				if(item == 4)
				{
					powerOption = 1;
					power = " Reboot     >Off";
					lcdUpdate = 1;
				}
				else if(item == 3)
				{
					color++;
					if(color > 7)
					{
						color = 7;
					}
					//LCD_color(color);
				}
				
				LCD_color(color);
				timer = 0;
				break;
			
			case Select: /* SELECT */
				
				if(item == 4)
				{
					if(powerOption)
					{
						LCD_wrap_printf("Powering Off...");
						system("sudo poweroff");
					}
					else
					{
						LCD_wrap_printf("Rebooting...");
						system("sudo reboot");
					}
					
				}
				else if(item == 5)
				{
					if(sendSIGUSR1())
					{
						color = 4;
					}
					else
					{
						color = 1;
					}
				}
				
				LCD_color(color);
				timer = 0;
				break;
			
			default:
			
				if(timer < 32000)
				{
					timer++;
				}
				else
				{
					LCD_color(0);
				}
				
		}
		
		
		/* Set item number */
		if(temp_item != item)
		{
			item = temp_item;
			lcdUpdate = 1;
		}
		
		
		
		/* Update each menu item as needed */
		switch(item)
		{
			case 0:
				strcpy(display_text, dev_menu[item].desc);
				break;
				
			case 1: /* BPM */
				/* open, read, and display the message from the FIFO */
				fd = open(bpm_pipe, O_RDONLY);
				if(read(fd, buf, 3) == 0)
				{
					buf = "---";
				}
				close(fd);
				strcpy(display_text, dev_menu[item].desc);
				strcat(display_text, buf);
				break;
				
			case 2: /* IP Address */
				fd = socket(AF_INET, SOCK_DGRAM, 0);

				/* IPv4 IP address */
				ifr.ifr_addr.sa_family = AF_INET;

				/*IP address attached to interface */
				strncpy(ifr.ifr_name , INTERFACE , IFNAMSIZ-1);

				ioctl(fd, SIOCGIFADDR, &ifr);

				close(fd);
				strcpy(display_text, dev_menu[item].desc);
				strcat(display_text, inet_ntoa(((struct sockaddr_in *)&ifr.ifr_addr)->sin_addr));
				//LCD_wrap_printf(display_text);
				break;
				
			case 4: /* Power Options */
				strcpy(display_text, dev_menu[item].desc);
				strcat(display_text, power);
				break;
				
			default: /* Title, Color */
				strcpy(display_text, dev_menu[item].desc);
				
		}
		
		
		/* checks if lcd needs to be updated or initialized */
		if(lcdUpdate || !init)
		{
			init = 1;
			lcdUpdate = 0;
			LCD_wrap_printf(display_text);
		}
		
		
		
    }
	
    return r;
}



static int set_color(const char *carg)
{
    if (!strcasecmp(carg, "1")) {
        return LCD_color(Black);
    } else if (!strcasecmp(carg, "2")) {
        return LCD_color(Red);
    } else if (!strcasecmp(carg, "3")) {
        return LCD_color(Yellow);
    } else if (!strcasecmp(carg, "4")) {
        return LCD_color(Green);
    } else if (!strcasecmp(carg, "5")) {
        return LCD_color(Cyan);
    } else if (!strcasecmp(carg, "6")) {
        return LCD_color(Blue);
    } else if (!strcasecmp(carg, "7")) {
        return LCD_color(Magenta);
    } else if (!strcasecmp(carg, "8")) {
        return LCD_color(White);
    } else {
        //fprintf(stderr, "set_color error: Invalid color.\n");
        LCD_color(White);
    }
    return 2;
}


static int set_cursor(const char *carg)
{
    if (!strcasecmp(carg, "Blink")) {
        return LCD_cmd(DISPLAY_SET | DISPLAY_ON | CURSOR_BLINK_ON);
    } else if (!strcasecmp(carg, "On")) {
        return LCD_cmd(DISPLAY_SET | DISPLAY_ON | CURSOR_ON);
    } else if (!strcasecmp(carg, "Off")) {
        return 0;
    } else {
        fprintf(stderr, "set_cursor error: Invalid option.\n");
    }
    return 3;
}

