clc;
clear;
close all;

%% Device Config
baud        = 57600;
port_name   = 'COM18';

f_s                    = 360;
words_per_write        = 255;
benchmarkdata_command  = uint32(hex2dec('FFFF0BFF'));

%% Load Data

use_dsp_data = false;
if(use_dsp_data)
    test_data = '../raw_data/sensor_dsp/2017-02-09_01.dat';     %file containing raw data
    D = int32(dlmread(test_data,'\n',1,0));
else
    test_data = '../raw_data/mit_bih/100.csv';     %file containing raw data
    D = csvread(test_data);
    D = D(:,2)';                   %CH1
    %D = D(:,3)';                   %CH2
    %D = (D(:,2)' + D(:,3)')/2;      %Average CH1 and CH2

    D = (D - min(D));
    D = int16(1024 * D/max(D));
end

N = length(D);

%% Open serial port
s = serial(port_name);
set(s, 'BaudRate', baud);
set(s, 'ByteOrder', 'bigEndian');
fopen(s);

for i = 0:(floor(N/words_per_write) - 1)
    fwrite(s, benchmarkdata_command, 'uint32');
    indices = (i*words_per_write + 1):((i+1)*words_per_write);
    for j = indices
        fwrite(s, D(j), 'int16');
        pause(1/f_s);
    end
end


%% Close connections
fclose(s);
delete(s);
clear s;