/*
 * ecg_buffer.h
 *
 *  Created on: Jan 31, 2017
 *      Author: edward
 */

#ifndef INC_ECG_BUFFER_H_
#define INC_ECG_BUFFER_H_

#include "ecg.h"

#include "rtecg.h"

/* Flags used to select which buffer gets transmitted over serial */
#define BUFFER_RAW 0
#define BUFFER_FILT 1

#define UART_READ_BUFFER_SIZE 1024
#define BENCHMARK_DATA_SIZE SENSOR_BUFFER*4

extern Uint16 ecg_sensor_buffer_index;
extern Uint16 uart_read_buffer_index;
extern Uint16 benchmark_data_buffer_index;
extern Bool input_buffer_ready;
extern char UART_read_buffer[UART_READ_BUFFER_SIZE];
extern Int32 ecg_sensor_buffer[SENSOR_BUFFER];
extern Int32 ecg_input_buffer[SENSOR_BUFFER];
extern Int16 benchmark_data_buffer[BENCHMARK_DATA_SIZE];


//Function: initialize_buffers
//initialize all buffers to 0
Bool initialize_buffers(void);


//Function: getCircularBufferIndex
//use to get circular buffer index
//Inputs:
//i: current index
//offset: relative index requested
//max_size: size of the circular buffer
//Return:
//new circular buffer index
Uint16 circular_buffer_index(Uint16 i, Int16 offset, Uint16 max_size);


//Function: copy_buffer
//Copy the contents of one buffer to another
//Inputs:
//source: buffer to copy from
//destination: buffer to copy to
//size: size of the buffers
//Return:
//return 'TRUE' if successful
Bool copy_buffer(Int32 source[], Int32 destination[], Uint16 max_size);


//Function: add_to_sensor_buffer
//Adds a value to the sensor buffer, increments index, and copies to input buffer if necessary
//Inputs:
//data: data to be added
//Return:
//return 'TRUE' if successful
Bool add_to_sensor_buffer(Int16 data);


#endif /* INC_ECG_BUFFER_H_ */
