/*
 * ecg_adc.c
 *
 *  Created on: Jan 31, 2017
 *      Author: edward
 */

#include "ecg_adc.h"
#include "ecg.h"

CSL_SarHandleObj	hSar;

Uint16 initialize_adc(void){

	//Channel configuration parameters are kept static to allow for switching channels to read multiple ADC pins
	static CSL_SarChConfig sarConfigParamCh1;
	static CSL_SarChConfig sarConfigParamCh4;
	static CSL_SarChConfig sarConfigParamCh5;

	//Setup SAR ADC
	status = SAR_init();
	if(CSL_SOK != status)
	{
		return ECG_ERROR;
	}
	//Open Channel
	status = SAR_chanOpen(&hSar, ECG_SENSOR_CHANNEL);
	if(CSL_SOK != status)
	{
		return ECG_ERROR;
	}

	//Setup channel 1 to read from GPAIN0
	//Select channel for ECG sensor (12), clear MULTICH to disable multi channel mode (11), set SNGLCONV to perform only a single conversion (10)
	sarConfigParamCh1.SARCTRL		= CSL_SAR_CHAN_1 << 12 | CSL_SAR_NO_DISCHARGE << 11 | CSL_SAR_SINGLE_CONVERSION  << 10;
	//Set clock divider for ADC clock
	sarConfigParamCh1.SARCLKCTRL	= ADC_CLKDIV & 0x7FFF;
	//clear STATUSMASK to enable status register, set PWRUPBIAS and SARPWRUP to power up device (12-13),
	//set REFBUFFEN to enable reference buffer for bandgap Vref (10), change REFLVSEL to select bandgap level (9),
	//clear REFAVDDSEL to use band gap Vref (8), disable touchscreen mode (4), clear AVDDMEAS to not measure internal analog voltage (3),
	//set GNDON to enable voltage divider to be 3.6v tolerant (1), set HALF to decrease ADC level by 2 before conversion (0)
	sarConfigParamCh1.SARPINCTRL	= 0 << 14 | 1 << 13 | 1 << 12 | 1 << 10 | CSL_SAR_REF_0_8v << 9 | 0 << 8 | 0 << 4 | 0 << 3 | 1 << 1 | 0 << 0;
	//clear PENIRQ to disable pen interrupt status (9), clear PENIRQEN to disable pen interrupt (8), clear GPOxEN to not use GPAINx as GPO GPAIN3-GPAIN0->(7-4),
	//clear GPOx to output GPAINx as GPO low GPAIN3-GPAIN0->(3-0)
	sarConfigParamCh1.SARGPOCTRL	= 0 << 9 | 0 << 8 | 0x0 << 4 | 0x0 << 0;

	//Setup channel 4 to read from GPAIN2
	//Select channel for ECG sensor (12), clear MULTICH to disable multi channel mode (11), set SNGLCONV to perform only a single conversion (10)
	sarConfigParamCh4.SARCTRL		= CSL_SAR_CHAN_4 << 12 | CSL_SAR_NO_DISCHARGE << 11 | CSL_SAR_SINGLE_CONVERSION  << 10;
	//Set clock divider for ADC clock
	sarConfigParamCh4.SARCLKCTRL	= ADC_CLKDIV & 0x7FFF;
	//clear STATUSMASK to enable status register, set PWRUPBIAS and SARPWRUP to power up device (12-13),
	//set REFBUFFEN to enable reference buffer for bandgap Vref (10), change REFLVSEL to select bandgap level (9),
	//clear REFAVDDSEL to use band gap Vref (8), disable touchscreen mode (4), clear AVDDMEAS to not measure internal analog voltage (3),
	//set GNDON to enable voltage divider to be 3.6v tolerant (1), set HALF to decrease ADC level by 2 before conversion (0)
	sarConfigParamCh4.SARPINCTRL	= 0 << 14 | 1 << 13 | 1 << 12 | 1 << 10 | CSL_SAR_REF_1v << 9 | 0 << 8 | 0 << 4 | 0 << 3 | 0 << 1 | 0 << 0;
	//clear PENIRQ to disable pen interrupt status (9), clear PENIRQEN to disable pen interrupt (8), clear GPOxEN to not use GPAINx as GPO GPAIN3-GPAIN0->(7-4),
	//clear GPOx to output GPAINx as GPO low GPAIN3-GPAIN0->(3-0)
	sarConfigParamCh4.SARGPOCTRL	= 0 << 9 | 0 << 8 | 0x0 << 4 | 0x0 << 0;

	//Setup channel 5 to read from GPAIN3
	//Select channel for ECG sensor (12), clear MULTICH to disable multi channel mode (11), set SNGLCONV to perform only a single conversion (10)
	sarConfigParamCh5.SARCTRL		= CSL_SAR_CHAN_5 << 12 | CSL_SAR_NO_DISCHARGE << 11 | CSL_SAR_SINGLE_CONVERSION  << 10;
	//Set clock divider for ADC clock
	sarConfigParamCh5.SARCLKCTRL	= ADC_CLKDIV & 0x7FFF;
	//clear STATUSMASK to enable status register, set PWRUPBIAS and SARPWRUP to power up device (12-13),
	//set REFBUFFEN to enable reference buffer for bandgap Vref (10), change REFLVSEL to select bandgap level (9),
	//clear REFAVDDSEL to use band gap Vref (8), disable touchscreen mode (4), clear AVDDMEAS to not measure internal analog voltage (3),
	//set GNDON to enable voltage divider to be 3.6v tolerant (1), set HALF to decrease ADC level by 2 before conversion (0)
	sarConfigParamCh5.SARPINCTRL	= 0 << 14 | 1 << 13 | 1 << 12 | 1 << 10 | CSL_SAR_REF_1v << 9 | 0 << 8 | 0 << 4 | 0 << 3 | 0 << 1 | 0 << 0;
	//clear PENIRQ to disable pen interrupt status (9), clear PENIRQEN to disable pen interrupt (8), clear GPOxEN to not use GPAINx as GPO GPAIN3-GPAIN0->(7-4),
	//clear GPOx to output GPAINx as GPO low GPAIN3-GPAIN0->(3-0)
	sarConfigParamCh5.SARGPOCTRL	= 0 << 9 | 0 << 8 | 0x0 << 4 | 0x0 << 0;

	if(ECG_SENSOR_CHANNEL == CSL_SAR_CHAN_1)
		status = SAR_chanConfig(&hSar, &sarConfigParamCh1);
	else if(ECG_SENSOR_CHANNEL == CSL_SAR_CHAN_4)
		status = SAR_chanConfig(&hSar, &sarConfigParamCh4);
	else if(ECG_SENSOR_CHANNEL == CSL_SAR_CHAN_5)
		status = SAR_chanConfig(&hSar, &sarConfigParamCh5);

	if(CSL_SOK != status)
	{
		return ECG_ERROR;
	}

	return ECG_SUCCESS;
}


Uint16 disable_adc(){
	status = SAR_chanClose(&hSar);
	status = SAR_deInit();

	return ECG_SUCCESS;
}
