#!/bin/bash
# Real-Time Electrocardiogram Monitoring
# Raspberry Pi crossbuild toolchain chroot helper script
# Calvin Walden
# 2017-01-24

GIT_DIR=./real-time-ecg
CHROOT_DIR=./debian-sid

# Check if this script is run as root.
if [[ $(id -u) -ne 0 ]]; then
    echo "start-chroot: This script must be run as root."
    exit
fi


if [ $# -eq 1 ]; then
    # If argument "umount", unmount directories
    if [ "$1" == "umount" ]; then
	echo "Unmounting $CHROOT_DIR/mnt/real-time-ecg."
	umount $CHROOT_DIR/mnt/real-time-ecg
	echo "Unmounting $CHROOT_DIR/proc."
	umount $CHROOT_DIR/proc
	echo "Unmounting $CHROOT_DIR/sys."
	umount $CHROOT_DIR/sys
	echo "Unmounting $CHROOT_DIR/dev."
	umount $CHROOT_DIR/dev
	echo "Unmounting $CHROOT_DIR/dev/pts."
	umount $CHROOT_DIR/dev/pts
    fi

else
    # Otherwise, mount filesystems in chroot direcotry and execute chroot.
    #      /mnt/real-time-ecg
    if ! [ "$(ls -A $CHROOT_DIR/mnt/real-time-ecg)" ]; then
	echo "Mounting git repository at $CHROOT_DIR/mnt/real-time-ecg."
	mount --bind $GIT_DIR $CHROOT_DIR/mnt/real-time-ecg
    else
	echo "Skipping mount git repository."
    fi

    #      /proc
    if ! [ "$(ls -A $CHROOT_DIR/proc)" ]; then
	echo "Mounting proc at chroot/proc."
	mount -t proc /proc $CHROOT_DIR/proc/
    else
	echo "Skipping mount proc."
    fi

    #      /sys
    if ! [ "$(ls -A $CHROOT_DIR/sys)" ]; then
	echo "Mounting sysfs at chroot/sys."
	mount -t sysfs sys $CHROOT_DIR/sys/
    else
	echo "Skipping mount sysfs."
    fi

    #      /dev
    if ! [ "$(ls -A $CHROOT_DIR/dev)" ]; then
	echo "Mounting dev at chroot/dev."
	mount -o bind /dev $CHROOT_DIR/dev/
    else
	echo "Skipping mount dev."
    fi

    #      /dev/pts
    if ! [ "$(ls -A $CHROOT_DIR/dev/pts)" ]; then
	echo "Mounting devpts at chroot/dev/pts."
	mount -t devpts pts $CHROOT_DIR/dev/pts/
    else
	echo "Skipping mount devpts."
    fi

    #    Change root
    chroot ./debian-sid
fi
